package ru.landgrafhomyak.serdha.api.v0

import kotlinx.datetime.Instant
import kotlinx.datetime.LocalDate
import kotlinx.datetime.LocalDateTime
import kotlinx.datetime.LocalTime
import ru.landgrafhomyak.serdha.api.v0.ddl.types.ColumnType

public interface Expression<RuntimeType, DatabaseType : ColumnType<RuntimeType>, OwnerBuilderUserExtension : Any> {
	public interface Builder<OwnerBuilderUserExtension : Any> {
		public fun <DatabaseType : ColumnType<*>> equals(
			left: Expression<*, DatabaseType, OwnerBuilderUserExtension>,
			right: Expression<*, DatabaseType, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun <DatabaseType : ColumnType<*>> notEquals(
			left: Expression<*, DatabaseType, OwnerBuilderUserExtension>,
			right: Expression<*, DatabaseType, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension> =
			this.logicalNot(this.equals(left, right))

		public fun isNull(
			e: Expression<*, ColumnType.Nullable<*, *>, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun isNotNull(
			e: Expression<*, ColumnType.Nullable<*, *>, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension> =
			this.logicalNot(this.isNull(e))

		public fun logicalAnd(
			left: Expression<*, ColumnType.BOOLEAN, OwnerBuilderUserExtension>,
			right: Expression<*, ColumnType.BOOLEAN, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun logicalOr(
			left: Expression<*, ColumnType.BOOLEAN, OwnerBuilderUserExtension>,
			right: Expression<*, ColumnType.BOOLEAN, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun logicalXor(
			left: Expression<*, ColumnType.BOOLEAN, OwnerBuilderUserExtension>,
			right: Expression<*, ColumnType.BOOLEAN, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension> =
			this.notEquals(left, right)

		public fun logicalNot(
			e: Expression<*, ColumnType.BOOLEAN, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLess(
			left: Expression<Byte, ColumnType.INT_S8, OwnerBuilderUserExtension>,
			right: Expression<Byte, ColumnType.INT_S8, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLess(
			left: Expression<UByte, ColumnType.INT_U8, OwnerBuilderUserExtension>,
			right: Expression<UByte, ColumnType.INT_U8, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLess(
			left: Expression<Short, ColumnType.INT_S16, OwnerBuilderUserExtension>,
			right: Expression<Short, ColumnType.INT_S16, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLess(
			left: Expression<UShort, ColumnType.INT_U16, OwnerBuilderUserExtension>,
			right: Expression<UShort, ColumnType.INT_U16, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLess(
			left: Expression<Int, ColumnType.INT_S32, OwnerBuilderUserExtension>,
			right: Expression<Int, ColumnType.INT_S32, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLess(
			left: Expression<UInt, ColumnType.INT_U32, OwnerBuilderUserExtension>,
			right: Expression<UInt, ColumnType.INT_U32, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLess(
			left: Expression<Long, ColumnType.INT_S64, OwnerBuilderUserExtension>,
			right: Expression<Long, ColumnType.INT_S64, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLess(
			left: Expression<ULong, ColumnType.INT_U64, OwnerBuilderUserExtension>,
			right: Expression<ULong, ColumnType.INT_U64, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLessOrEquals(
			left: Expression<Byte, ColumnType.INT_S8, OwnerBuilderUserExtension>,
			right: Expression<Byte, ColumnType.INT_S8, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLessOrEquals(
			left: Expression<UByte, ColumnType.INT_U8, OwnerBuilderUserExtension>,
			right: Expression<UByte, ColumnType.INT_U8, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLessOrEquals(
			left: Expression<Short, ColumnType.INT_S16, OwnerBuilderUserExtension>,
			right: Expression<Short, ColumnType.INT_S16, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLessOrEquals(
			left: Expression<UShort, ColumnType.INT_U16, OwnerBuilderUserExtension>,
			right: Expression<UShort, ColumnType.INT_U16, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLessOrEquals(
			left: Expression<Int, ColumnType.INT_S32, OwnerBuilderUserExtension>,
			right: Expression<Int, ColumnType.INT_S32, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLessOrEquals(
			left: Expression<UInt, ColumnType.INT_U32, OwnerBuilderUserExtension>,
			right: Expression<UInt, ColumnType.INT_U32, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLessOrEquals(
			left: Expression<Long, ColumnType.INT_S64, OwnerBuilderUserExtension>,
			right: Expression<Long, ColumnType.INT_S64, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersLessOrEquals(
			left: Expression<ULong, ColumnType.INT_U64, OwnerBuilderUserExtension>,
			right: Expression<ULong, ColumnType.INT_U64, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreater(
			left: Expression<Byte, ColumnType.INT_S8, OwnerBuilderUserExtension>,
			right: Expression<Byte, ColumnType.INT_S8, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreater(
			left: Expression<UByte, ColumnType.INT_U8, OwnerBuilderUserExtension>,
			right: Expression<UByte, ColumnType.INT_U8, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreater(
			left: Expression<Short, ColumnType.INT_S16, OwnerBuilderUserExtension>,
			right: Expression<Short, ColumnType.INT_S16, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreater(
			left: Expression<UShort, ColumnType.INT_U16, OwnerBuilderUserExtension>,
			right: Expression<UShort, ColumnType.INT_U16, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreater(
			left: Expression<Int, ColumnType.INT_S32, OwnerBuilderUserExtension>,
			right: Expression<Int, ColumnType.INT_S32, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreater(
			left: Expression<UInt, ColumnType.INT_U32, OwnerBuilderUserExtension>,
			right: Expression<UInt, ColumnType.INT_U32, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreater(
			left: Expression<Long, ColumnType.INT_S64, OwnerBuilderUserExtension>,
			right: Expression<Long, ColumnType.INT_S64, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreater(
			left: Expression<ULong, ColumnType.INT_U64, OwnerBuilderUserExtension>,
			right: Expression<ULong, ColumnType.INT_U64, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreaterOrEquals(
			left: Expression<Byte, ColumnType.INT_S8, OwnerBuilderUserExtension>,
			right: Expression<Byte, ColumnType.INT_S8, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreaterOrEquals(
			left: Expression<UByte, ColumnType.INT_U8, OwnerBuilderUserExtension>,
			right: Expression<UByte, ColumnType.INT_U8, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreaterOrEquals(
			left: Expression<Short, ColumnType.INT_S16, OwnerBuilderUserExtension>,
			right: Expression<Short, ColumnType.INT_S16, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreaterOrEquals(
			left: Expression<UShort, ColumnType.INT_U16, OwnerBuilderUserExtension>,
			right: Expression<UShort, ColumnType.INT_U16, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreaterOrEquals(
			left: Expression<Int, ColumnType.INT_S32, OwnerBuilderUserExtension>,
			right: Expression<Int, ColumnType.INT_S32, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreaterOrEquals(
			left: Expression<UInt, ColumnType.INT_U32, OwnerBuilderUserExtension>,
			right: Expression<UInt, ColumnType.INT_U32, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreaterOrEquals(
			left: Expression<Long, ColumnType.INT_S64, OwnerBuilderUserExtension>,
			right: Expression<Long, ColumnType.INT_S64, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpIntegersGreaterOrEquals(
			left: Expression<ULong, ColumnType.INT_U64, OwnerBuilderUserExtension>,
			right: Expression<ULong, ColumnType.INT_U64, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun extendIntegerToS16(
			e: Expression<Byte, ColumnType.INT_S8, OwnerBuilderUserExtension>,
		): Expression<Short, ColumnType.INT_S16, OwnerBuilderUserExtension>

		public fun extendIntegerToS16(
			e: Expression<UByte, ColumnType.INT_U8, OwnerBuilderUserExtension>,
		): Expression<Short, ColumnType.INT_S16, OwnerBuilderUserExtension>

		public fun extendIntegerToU16(
			e: Expression<UByte, ColumnType.INT_U8, OwnerBuilderUserExtension>,
		): Expression<UShort, ColumnType.INT_U16, OwnerBuilderUserExtension>

		public fun extendIntegerToS32(
			e: Expression<Byte, ColumnType.INT_S8, OwnerBuilderUserExtension>,
		): Expression<Int, ColumnType.INT_S32, OwnerBuilderUserExtension>

		public fun extendIntegerToS32(
			e: Expression<UByte, ColumnType.INT_U8, OwnerBuilderUserExtension>,
		): Expression<Int, ColumnType.INT_S32, OwnerBuilderUserExtension>

		public fun extendIntegerToS32(
			e: Expression<Short, ColumnType.INT_S16, OwnerBuilderUserExtension>,
		): Expression<Int, ColumnType.INT_S32, OwnerBuilderUserExtension>

		public fun extendIntegerToS32(
			e: Expression<UShort, ColumnType.INT_U16, OwnerBuilderUserExtension>,
		): Expression<Int, ColumnType.INT_S32, OwnerBuilderUserExtension>

		public fun extendIntegerToU32(
			e: Expression<UByte, ColumnType.INT_U8, OwnerBuilderUserExtension>,
		): Expression<UInt, ColumnType.INT_U32, OwnerBuilderUserExtension>

		public fun extendIntegerToU32(
			e: Expression<UShort, ColumnType.INT_U16, OwnerBuilderUserExtension>,
		): Expression<UInt, ColumnType.INT_U32, OwnerBuilderUserExtension>

		public fun extendIntegerToS64(
			e: Expression<Byte, ColumnType.INT_S8, OwnerBuilderUserExtension>,
		): Expression<Long, ColumnType.INT_S64, OwnerBuilderUserExtension>

		public fun extendIntegerToS64(
			e: Expression<UByte, ColumnType.INT_U8, OwnerBuilderUserExtension>,
		): Expression<Long, ColumnType.INT_S64, OwnerBuilderUserExtension>

		public fun extendIntegerToS64(
			e: Expression<Short, ColumnType.INT_S16, OwnerBuilderUserExtension>,
		): Expression<Long, ColumnType.INT_S64, OwnerBuilderUserExtension>

		public fun extendIntegerToS64(
			e: Expression<UShort, ColumnType.INT_U16, OwnerBuilderUserExtension>,
		): Expression<Long, ColumnType.INT_S64, OwnerBuilderUserExtension>

		public fun extendIntegerToS64(
			e: Expression<Int, ColumnType.INT_S32, OwnerBuilderUserExtension>,
		): Expression<Long, ColumnType.INT_S64, OwnerBuilderUserExtension>

		public fun extendIntegerToS64(
			e: Expression<UInt, ColumnType.INT_U32, OwnerBuilderUserExtension>,
		): Expression<Long, ColumnType.INT_S64, OwnerBuilderUserExtension>

		public fun extendIntegerToU64(
			e: Expression<UByte, ColumnType.INT_U8, OwnerBuilderUserExtension>,
		): Expression<ULong, ColumnType.INT_U64, OwnerBuilderUserExtension>

		public fun extendIntegerToU64(
			e: Expression<UShort, ColumnType.INT_U16, OwnerBuilderUserExtension>,
		): Expression<ULong, ColumnType.INT_U64, OwnerBuilderUserExtension>

		public fun extendIntegerToU64(
			e: Expression<UInt, ColumnType.INT_U32, OwnerBuilderUserExtension>,
		): Expression<ULong, ColumnType.INT_U64, OwnerBuilderUserExtension>

		public fun extendFractionalToDouble(
			e: Expression<Float, ColumnType.FLOAT_32, OwnerBuilderUserExtension>
		): Expression<Double, ColumnType.FLOAT_64, OwnerBuilderUserExtension>

		public fun cmpFractionalLess(
			left: Expression<Float, ColumnType.FLOAT_32, OwnerBuilderUserExtension>,
			right: Expression<Float, ColumnType.FLOAT_32, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpFractionalLess(
			left: Expression<Double, ColumnType.FLOAT_64, OwnerBuilderUserExtension>,
			right: Expression<Double, ColumnType.FLOAT_64, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpFractionalLessOrEquals(
			left: Expression<Float, ColumnType.FLOAT_32, OwnerBuilderUserExtension>,
			right: Expression<Float, ColumnType.FLOAT_32, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpFractionalLessOrEquals(
			left: Expression<Double, ColumnType.FLOAT_64, OwnerBuilderUserExtension>,
			right: Expression<Double, ColumnType.FLOAT_64, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpFractionalGreater(
			left: Expression<Float, ColumnType.FLOAT_32, OwnerBuilderUserExtension>,
			right: Expression<Float, ColumnType.FLOAT_32, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpFractionalGreater(
			left: Expression<Double, ColumnType.FLOAT_64, OwnerBuilderUserExtension>,
			right: Expression<Double, ColumnType.FLOAT_64, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpFractionalGreaterOrEquals(
			left: Expression<Float, ColumnType.FLOAT_32, OwnerBuilderUserExtension>,
			right: Expression<Float, ColumnType.FLOAT_32, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpFractionalGreaterOrEquals(
			left: Expression<Double, ColumnType.FLOAT_64, OwnerBuilderUserExtension>,
			right: Expression<Double, ColumnType.FLOAT_64, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalDateEarlier(
			left: Expression<LocalDate, ColumnType.LOCAL_DATE, OwnerBuilderUserExtension>,
			right: Expression<LocalDate, ColumnType.LOCAL_DATE, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalDateEarlierOrSimultaneous(
			left: Expression<LocalDate, ColumnType.LOCAL_DATE, OwnerBuilderUserExtension>,
			right: Expression<LocalDate, ColumnType.LOCAL_DATE, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalDateLater(
			left: Expression<LocalDate, ColumnType.LOCAL_DATE, OwnerBuilderUserExtension>,
			right: Expression<LocalDate, ColumnType.LOCAL_DATE, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalDateLaterOrSimultaneous(
			left: Expression<LocalDate, ColumnType.LOCAL_DATE, OwnerBuilderUserExtension>,
			right: Expression<LocalDate, ColumnType.LOCAL_DATE, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalTimeEarlier(
			left: Expression<LocalTime, ColumnType.LOCAL_TIME, OwnerBuilderUserExtension>,
			right: Expression<LocalTime, ColumnType.LOCAL_TIME, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalTimeEarlierOrSimultaneous(
			left: Expression<LocalTime, ColumnType.LOCAL_TIME, OwnerBuilderUserExtension>,
			right: Expression<LocalTime, ColumnType.LOCAL_TIME, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalTimeLater(
			left: Expression<LocalTime, ColumnType.LOCAL_TIME, OwnerBuilderUserExtension>,
			right: Expression<LocalTime, ColumnType.LOCAL_TIME, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalTimeLaterOrSimultaneous(
			left: Expression<LocalTime, ColumnType.LOCAL_TIME, OwnerBuilderUserExtension>,
			right: Expression<LocalTime, ColumnType.LOCAL_TIME, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalDateTimeEarlier(
			left: Expression<LocalDateTime, ColumnType.LOCAL_DATETIME, OwnerBuilderUserExtension>,
			right: Expression<LocalDateTime, ColumnType.LOCAL_DATETIME, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalDateTimeEarlierOrSimultaneous(
			left: Expression<LocalDateTime, ColumnType.LOCAL_DATETIME, OwnerBuilderUserExtension>,
			right: Expression<LocalDateTime, ColumnType.LOCAL_DATETIME, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalDateTimeLater(
			left: Expression<LocalDateTime, ColumnType.LOCAL_DATETIME, OwnerBuilderUserExtension>,
			right: Expression<LocalDateTime, ColumnType.LOCAL_DATETIME, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpLocalDateTimeLaterOrSimultaneous(
			left: Expression<LocalDateTime, ColumnType.LOCAL_DATETIME, OwnerBuilderUserExtension>,
			right: Expression<LocalDateTime, ColumnType.LOCAL_DATETIME, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpInstantEarlier(
			left: Expression<Instant, ColumnType.INSTANT, OwnerBuilderUserExtension>,
			right: Expression<Instant, ColumnType.INSTANT, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpInstantEarlierOrSimultaneous(
			left: Expression<Instant, ColumnType.INSTANT, OwnerBuilderUserExtension>,
			right: Expression<Instant, ColumnType.INSTANT, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpInstantLater(
			left: Expression<Instant, ColumnType.INSTANT, OwnerBuilderUserExtension>,
			right: Expression<Instant, ColumnType.INSTANT, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun cmpInstantLaterOrSimultaneous(
			left: Expression<Instant, ColumnType.INSTANT, OwnerBuilderUserExtension>,
			right: Expression<Instant, ColumnType.INSTANT, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun dateComponentOf(
			dt: Expression<LocalDateTime, ColumnType.LOCAL_DATETIME, OwnerBuilderUserExtension>,
		): Expression<LocalDate, ColumnType.LOCAL_DATE, OwnerBuilderUserExtension>

		public fun timeComponentOf(
			dt: Expression<LocalDateTime, ColumnType.LOCAL_DATETIME, OwnerBuilderUserExtension>,
		): Expression<LocalTime, ColumnType.LOCAL_TIME, OwnerBuilderUserExtension>


		public fun joinDateTime(
			date: Expression<LocalDate, ColumnType.LOCAL_DATE, OwnerBuilderUserExtension>,
			time: Expression<LocalTime, ColumnType.LOCAL_TIME, OwnerBuilderUserExtension>,
		): Expression<LocalDateTime, ColumnType.LOCAL_DATETIME, OwnerBuilderUserExtension>


		public fun constantOf(v: String): Expression<String, ColumnType.STRING, OwnerBuilderUserExtension>
		public fun constantOf(v: Boolean): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>
		public fun constantOf(v: Byte): Expression<Byte, ColumnType.INT_S8, OwnerBuilderUserExtension>
		public fun constantOf(v: UByte): Expression<UByte, ColumnType.INT_U8, OwnerBuilderUserExtension>
		public fun constantOf(v: Short): Expression<Short, ColumnType.INT_S16, OwnerBuilderUserExtension>
		public fun constantOf(v: UShort): Expression<UShort, ColumnType.INT_U16, OwnerBuilderUserExtension>
		public fun constantOf(v: Int): Expression<Int, ColumnType.INT_S32, OwnerBuilderUserExtension>
		public fun constantOf(v: UInt): Expression<UInt, ColumnType.INT_U32, OwnerBuilderUserExtension>
		public fun constantOf(v: Long): Expression<Long, ColumnType.INT_S64, OwnerBuilderUserExtension>
		public fun constantOf(v: ULong): Expression<ULong, ColumnType.INT_U64, OwnerBuilderUserExtension>
		public fun constantOf(v: Float): Expression<Float, ColumnType.FLOAT_32, OwnerBuilderUserExtension>
		public fun constantOf(v: Double): Expression<Double, ColumnType.FLOAT_64, OwnerBuilderUserExtension>
		public fun constantOf(v: LocalDate): Expression<LocalDate, ColumnType.LOCAL_DATE, OwnerBuilderUserExtension>
		public fun constantOf(v: LocalTime): Expression<LocalTime, ColumnType.LOCAL_TIME, OwnerBuilderUserExtension>
		public fun constantOf(v: LocalDateTime): Expression<LocalDateTime, ColumnType.LOCAL_DATETIME, OwnerBuilderUserExtension>
		public fun constantOf(v: Instant): Expression<Instant, ColumnType.INSTANT, OwnerBuilderUserExtension>

		@OptIn(ExperimentalUnsignedTypes::class)
		public fun constantOf(v: ByteArray): Expression<UByteArray, ColumnType.BYTE_ARRAY, OwnerBuilderUserExtension>

		public fun <ArgRuntimeType : Any, ArgDatabaseType : ColumnType<ArgRuntimeType>, RetRuntimeType, RetDatabaseType : ColumnType<RetRuntimeType>> ifNull(
			nullable: Expression<ArgRuntimeType?, ColumnType.Nullable<ArgRuntimeType, ArgDatabaseType>, OwnerBuilderUserExtension>,
			onValue: (Expression<ArgRuntimeType, ArgDatabaseType, OwnerBuilderUserExtension>) -> Expression<RetRuntimeType, RetDatabaseType, OwnerBuilderUserExtension>,
			onNull: () -> Expression<RetRuntimeType, RetDatabaseType, OwnerBuilderUserExtension>
		): Expression<RetRuntimeType, RetDatabaseType, OwnerBuilderUserExtension>

		public fun <ArgRuntimeType : Any, ArgDatabaseType : ColumnType<ArgRuntimeType>> notNullAnd(
			nullable: Expression<ArgRuntimeType?, ColumnType.Nullable<ArgRuntimeType, ArgDatabaseType>, OwnerBuilderUserExtension>,
			onValue: (Expression<ArgRuntimeType, ArgDatabaseType, OwnerBuilderUserExtension>) -> Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>

		public fun <ArgRuntimeType : Any, ArgDatabaseType : ColumnType<ArgRuntimeType>> nullOr(
			nullable: Expression<ArgRuntimeType?, ColumnType.Nullable<ArgRuntimeType, ArgDatabaseType>, OwnerBuilderUserExtension>,
			onValue: (Expression<ArgRuntimeType, ArgDatabaseType, OwnerBuilderUserExtension>) -> Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>,
		): Expression<Boolean, ColumnType.BOOLEAN, OwnerBuilderUserExtension>
	}
}